@@ -792,7 +792,11 @@ gegl_node_invalidated (GeglNode            *node,
 
   rects = g_hash_table_new_full (NULL, NULL, NULL, g_free);
 
+#if GLIB_CHECK_VERSION(2,68,0)
   g_hash_table_insert (rects, node, g_memdup2 (rect, sizeof (GeglRectangle)));
+#else
+  g_hash_table_insert (rects, node, g_memdup (rect, sizeof (GeglRectangle)));
+#endif
 
   visitor = gegl_callback_visitor_new (gegl_node_invalidated_invalidate_node,
                                        rects);
@@ -880,6 +884,13 @@ gegl_node_connect (GeglNode    *source,
     return FALSE;
 
   GeglPad *pad_sink = gegl_node_get_pad (sink, sink_pad_name);
+  if (pad_sink == NULL)
+    {
+      g_warning ("%s: the sink node '%s' doesn't have a pad named '%s', bailing.\n",
+                 G_STRFUNC, gegl_node_get_debug_name (sink), sink_pad_name);
+      return FALSE;
+    }
+
   if (!gegl_pad_is_input (pad_sink))
   {
     { GeglNode *tmp = source;
