@@ -41,7 +41,11 @@ struct _GeglColorPrivate
 
   union
   {
-    guint8  pixel[40];
+    /* Some babl code (SSE2 codepath in particular) requires source address to
+     * be 16-byte aligned and would crash otherwise.
+     * See: https://gitlab.gnome.org/GNOME/gegl/-/merge_requests/142
+     */
+    guint8  pixel[48] __attribute__((aligned(16)));
     gdouble alignment;
   };
 };
@@ -310,6 +314,48 @@ gegl_color_get_pixel (GeglColor   *color,
                 color->priv->pixel, pixel, 1);
 }
 
+void
+gegl_color_set_bytes (GeglColor  *color,
+                      const Babl *format,
+                      GBytes     *bytes)
+{
+  gint bpp;
+
+  g_return_if_fail (GEGL_IS_COLOR (color));
+  g_return_if_fail (format);
+  g_return_if_fail (bytes);
+
+  bpp = babl_format_get_bytes_per_pixel (format);
+  g_return_if_fail (g_bytes_get_size (bytes) == bpp);
+
+  if (bpp <= sizeof (color->priv->pixel))
+    color->priv->format = format;
+  else
+    color->priv->format = gegl_babl_rgba_linear_float ();
+
+  babl_process (babl_fish (format, color->priv->format),
+                g_bytes_get_data (bytes, NULL), color->priv->pixel, 1);
+}
+
+GBytes *
+gegl_color_get_bytes (GeglColor  *color,
+                      const Babl *format)
+{
+  guint8 *data;
+  gint    bpp;
+
+  g_return_val_if_fail (GEGL_IS_COLOR (color), NULL);
+  g_return_val_if_fail (format, NULL);
+
+  bpp  = babl_format_get_bytes_per_pixel (format);
+  data = g_malloc0 (bpp);
+
+  babl_process (babl_fish (color->priv->format, format),
+                color->priv->pixel, data, 1);
+
+  return g_bytes_new_take (data, bpp);
+}
+
 void
 gegl_color_set_rgba (GeglColor *self,
                      gdouble    r,
@@ -343,6 +389,99 @@ gegl_color_get_rgba (GeglColor *self,
   if (a) *a = rgba[3];
 }
 
+void
+gegl_color_set_rgba_with_space (GeglColor  *self,
+                                gdouble     r,
+                                gdouble     g,
+                                gdouble     b,
+                                gdouble     a,
+                                const Babl *space)
+{
+  const Babl   *format  = babl_format_with_space ("R'G'B'A float", space);
+  const gfloat  rgba[4] = {r, g, b, a};
+
+  space = babl_format_get_space (format);
+
+  g_return_if_fail (GEGL_IS_COLOR (self));
+#if BABL_MINOR_VERSION > 1 || (BABL_MINOR_VERSION == 1 && BABL_MICRO_VERSION >= 107)
+  g_return_if_fail (space == NULL || babl_space_is_rgb (space));
+#else
+  g_return_if_fail (space == NULL || (! babl_space_is_cmyk (space) && ! babl_space_is_gray (space)));
+#endif
+
+  gegl_color_set_pixel (self, format, rgba);
+}
+
+void
+gegl_color_get_rgba_with_space (GeglColor  *self,
+                                gdouble    *r,
+                                gdouble    *g,
+                                gdouble    *b,
+                                gdouble    *a,
+                                const Babl *space)
+{
+  const Babl *format  = babl_format_with_space ("R'G'B'A float", space);
+  gfloat      rgba[4];
+
+  space = babl_format_get_space (format);
+
+  g_return_if_fail (GEGL_IS_COLOR (self));
+#if BABL_MINOR_VERSION > 1 || (BABL_MINOR_VERSION == 1 && BABL_MICRO_VERSION >= 107)
+  g_return_if_fail (space == NULL || babl_space_is_rgb (space));
+#else
+  g_return_if_fail (space == NULL || (! babl_space_is_cmyk (space) && ! babl_space_is_gray (space)));
+#endif
+
+  gegl_color_get_pixel (self, format, rgba);
+
+  if (r) *r = rgba[0];
+  if (g) *g = rgba[1];
+  if (b) *b = rgba[2];
+  if (a) *a = rgba[3];
+}
+
+void
+gegl_color_set_cmyk (GeglColor  *self,
+                     gdouble     c,
+                     gdouble     m,
+                     gdouble     y,
+                     gdouble     k,
+                     gdouble     a,
+                     const Babl *space)
+{
+  const Babl   *format  = babl_format_with_space ("CMYK float", space);
+  const gfloat  cmyk[5] = {c, m, y, k, a};
+
+  g_return_if_fail (GEGL_IS_COLOR (self));
+  g_return_if_fail (space == NULL || babl_format_get_space (format));
+
+  gegl_color_set_pixel (self, format, cmyk);
+}
+
+void
+gegl_color_get_cmyk (GeglColor  *self,
+                     gdouble    *c,
+                     gdouble    *m,
+                     gdouble    *y,
+                     gdouble    *k,
+                     gdouble    *a,
+                     const Babl *space)
+{
+  const Babl *format  = babl_format_with_space ("CMYK float", space);
+  gfloat      cmyk[5];
+
+  g_return_if_fail (GEGL_IS_COLOR (self));
+  g_return_if_fail (space == NULL || babl_space_is_cmyk (babl_format_get_space (format)));
+
+  gegl_color_get_pixel (self, format, cmyk);
+
+  if (c) *c = cmyk[0];
+  if (m) *m = cmyk[1];
+  if (y) *y = cmyk[2];
+  if (k) *k = cmyk[2];
+  if (a) *a = cmyk[3];
+}
+
 static void
 gegl_color_set_from_string (GeglColor   *self,
                             const gchar *color_string)
@@ -588,6 +727,21 @@ gegl_param_color_set_default (GParamSpec *param_spec,
     g_value_take_object (value, gegl_color_duplicate (gegl_color->default_color));
 }
 
+static gint
+gegl_param_color_cmp (GParamSpec   *param_spec,
+                      const GValue *value1,
+                      const GValue *value2)
+{
+  GeglColor *color1 = g_value_get_object (value1);
+  GeglColor *color2 = g_value_get_object (value2);
+
+  if (color1->priv->format != color2->priv->format)
+    return 1;
+  else
+    return memcmp (color1->priv->pixel, color2->priv->pixel,
+                   babl_format_get_bytes_per_pixel (color1->priv->format));
+}
+
 GType
 gegl_param_color_get_type (void)
 {
@@ -603,7 +757,7 @@ gegl_param_color_get_type (void)
         gegl_param_color_finalize,
         gegl_param_color_set_default,
         NULL,
-        NULL
+        gegl_param_color_cmp
       };
       param_color_type_info.value_type = GEGL_TYPE_COLOR;
 
